'LCD routines
'  INITIALIZE_LCD
'  PRINT (the_line)
'  POSITION (LINE_NUMBER, HOR_POSITION) | CHAR_LOCATION
'  SEND_CHAR (DISPLAY_CHAR)
'  SEND_CHAR (DISPLAY_CHAR)
'  PRINT_DEC (VALUE) | TEST_VALUE
'  PRINT_HEX (VALUE, DIGITS)
'  PRINT_BIN (VALUE, DIGITS)
'  CLEAR
'  HOME
'  SPACE
' 
CON                             'all the constants used by all the METHODS herein have to be listed here               
  DataBit0   = 16               'Data uses 8 bits from here
  DataBit7   = 23               'to here                  
  RegSelect  = 8                'The three control lines           
  ReadWrite  = 9                '            
  Enable     = 10               '
  
  waitPeriod =4_000_000         {set the wait period, about 1/10 secs for the LEDs}
  high       =1                 {define the High state}
  low        =0                 {define the Low state}
  _clkmode = xtal1 + pll8x
  _clkfreq = 100_000            'makes the Osc Freq 40_000_000

VAR                             'these are the variables we will use. They are available for all uses
   byte  temp                   'for temporary storage
   byte  index                  'for temporary storage
   long  delay                  'for temporary storage
   long  pulse                  'for temporary storage

                                                                                                                                                                                            
PUB INITIALIZE_LCD                             'The addressses and data used here are
  waitcnt(50_000+cnt)                          'specified in the Hitachi data sheet for the
  DIRA[RegSelect]~~                            'display. YOU MUST CHECK THIS FOR YOURSELF.
  DIRA[ReadWrite]~~                            ' ** changed from original which had fixed pins
  DIRA[Enable]~~                               ' **   "
  OUTA[RegSelect] := 0                         'these three lines are specfied so we can write
  OUTA[ReadWrite] := 0                         'the intial set up bits for the LCD
  OUTA[Enable]    := 0                         'See Hitachi HD44780 data sheet                                    
  SEND_INSTRUCTION (%0011_1000)                'Sets DL=8 bits, N=2 lines, F=5x7 fon
  SEND_INSTRUCTION (%0000_0001)                'clears the LCD
  'waitcnt(1000 +cnt)
  SEND_INSTRUCTION (%0000_1100)                'Display on, Cursor off, Blink off                                            
  SEND_INSTRUCTION (%0000_0110)                'Move Cursor, Do not shift display 
  POSITION (1,1)
                            
PUB PRINT (the_line)                           'This routine handles more than one Char at a time
'called as PRINT(string("the_line"))           '"the_line" contains the pointer to the line. Line is
'because we have to point to the line          'zero terminated but we will not use that.  We will
                                               'use the string size instead. Easier to understand
  index:=0                                     'Reset the counter we are using to count chars sent
  repeat                                       'repeat for all chars in the list
    temp:= byte[the_line][index++]             'temp contains the char/byte pointed to by the index
    SEND_CHAR (temp)                           'send the 'pointed to' char to the LCD
  while index<strsize(the_line)                'keep doing it till the last char is sent 

PUB POSITION (LINE_NUMBER, HOR_POSITION) | CHAR_LOCATION    'Position the cursor at location
  'Horizontal Position : 1 to 16              'specified by the two numbers
  'Line Number : 1 or 2        
  CHAR_LOCATION := (LINE_NUMBER-1) * 64       'figure location. See Hitachi HD44780 data sheet
  CHAR_LOCATION += (HOR_POSITION-1) + 128     'figure location. See Hitachi HD44780 data sheet
  SEND_INSTRUCTION (CHAR_LOCATION)            'send the instruction to position cursor

PUB SEND_CHAR (DISPLAY_CHAR)                  'set up for writing to the display    
  CHECK_BUSY                                  'wait for busy bit to clear before sending
  OUTA[ReadWrite] := 0                        'Set up to read busy bit
  OUTA[RegSelect] := 1                        'Set up to read busy bit
  OUTA[Enable]    := 1                        'Set up to toggle bit H>L
  OUTA[DataBit7..DataBit0] := DISPLAY_CHAR    'Ready to SEND data in
  OUTA[Enable]    := 0                        'Toggle the bit H>L
  
PUB CHECK_BUSY | BUSY_BIT                     'routine to check busy bit
  OUTA[ReadWrite] := 1                        'Set to read the busy bit         
  OUTA[RegSelect] := 0                        'Set to read the busy bit
  DIRA[DataBit7..DataBit0] := %0000_0000      'Set the entire port to be an input         
  REPEAT                                      'Keep doing it till clear
    OUTA[Enable]  := 1                        'set to 1 to get ready to toggle H>L this bit
    BUSY_BIT := INA[DataBit7]                 'the busybit is bit 7 of the byte read
                                              'INA is the 32 input pins on the PROP and we
                                              'are reading data bit 7 which is on pin 15!
    OUTA[Enable]  := 0                        'make the enable bit go low for H>L toggle
  WHILE (BUSY_BIT == 1)                       'do it as long as the busy bit is 1
  DIRA[DataBit7..DataBit0] := %1111_1111      'done, so set the data port back to outputs
                                                                        
PUB SEND_INSTRUCTION (DISPLAY_DATA)           'set up for writing instructions
  DIRA[DataBit0..DataBit7]~~                  ' **corrected so it is not fixed to I/O pins
  CHECK_BUSY                                  'wait for busy bit to clear before sending
  OUTA[ReadWrite] := 0                        'Set up to read busy bit         
  OUTA[RegSelect] := 0                        'Set up to read busy bit         
  OUTA[Enable]    := 1                        'Set up to toggle bit H>L
  OUTA[DataBit7..DataBit0] := DISPLAY_DATA    'Ready to READ data in
  OUTA[Enable]    := 0                        'Toggle the bit H>L       '
                                              '
PUB PRINT_DEC (VALUE) | TEST_VALUE            'for printing values in decimal format
  IF (VALUE < 0)                              'if it is a negative value             
    -VALUE                                    'chamge it to a positive
    SEND_CHAR("-")                            'and print a - sign on the LCD
                                              '
  TEST_VALUE := 1_000_000_000                 'we get individual digits by comparing to this
                                              'value and then dividing by 10 to get the next value
  REPEAT 10                                   'There are 10 digits maximum in our system
    IF (VALUE => TEST_VALUE)                  'see if our number is bigger than testValue
      SEND_CHAR(VALUE / TEST_VALUE + "0")     'if it is divide to get the digit 
      VALUE //= TEST_VALUE                    'figure the next value for the next digit
      RESULT~~                                'result of what just did so we can pass it on below
    ELSEIF (RESULT OR TEST_VALUE == 1)        'if the result was a 1 then division was even
      SEND_CHAR("0")                          'so we sent out a zero
    TEST_VALUE /= 10                          'we divide by 10 to test for the next digit
                                              '                                             
PUB PRINT_HEX (VALUE, DIGITS)                 'for printing values in HEX format
  VALUE <<= (8 - DIGITS) << 2                 'you can specify up to 8 digits or FFFFFFFF max
  REPEAT DIGITS                               'do each digit
    SEND_CHAR(LOOKUPZ((VALUE <-= 4) & $F : "0".."9", "A".."F")) 'use loookup table to select character
                                              '
PUB PRINT_BIN (VALUE, DIGITS)                 'for printing values in BINARY format
  VALUE <<= 32 - DIGITS                       '32 binary digits is the max for our system
  REPEAT DIGITS                               'Repeat for each digit desired
    SEND_CHAR((VALUE <-= 1) & 1 + "0")        'send a 1 or a 0
    
PUB CLEAR                                      'Clear the LCD display and go home
  SEND_INSTRUCTION (%0000_0001)                'This is the clear screen and go home command
  waitcnt(1000000+cnt)
PUB HOME                                       'go to position 1,1.  
  SEND_INSTRUCTION (%0000_0011)                'Not cleared

PUB SPACE (qty)                                'Prints spaces, for between displayed numbers etc
  repeat (qty)
    PRINT(STRING(" "))
                   